<?php
namespace Teknisa\CRM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\CRM\Factory\TicketBotFactory;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DTO;

class TicketBotService
{
    protected $entityManager;
    protected $environment;
    protected $ticketBotFactory;
    
    public function __construct(EntityManager $entityManager, $environment, TicketBotFactory $ticketBotFactory, $enviarMensagemController, $sessionManager) {
        $this->entityManager = $entityManager;
        $this->ticketBotFactory = $ticketBotFactory;
        $this->environment = $environment;
        $this->enviarMensagemController = $enviarMensagemController;
        $this->sessionManager = $sessionManager;
    }
    
    /**
     *  Lê os emails da caixa de email, cria os tickets e notifica o solicitante de suia abertura.
     * 
     *  @param PHPImap\Mailbox  $mailBox            Instância da classe da biblioteca IMAP.
     *  @param array            $mailsIds           Array de IDS de emails da caixa de email.
     * 
     *  @return array                               Array mostrando o Id dos tickets criados e uma descrição.
    */
    public function getMails($mailBox, $mailsIds){
        //Ordena os emails - mais recentes primeiro.
        rsort($mailsIds);
        
        $count = 0;
        $tickets = array();
        $successMails = array();
        $errorMails = array();
        $errorMessage = array();
        
        foreach($mailsIds as $mailId){
            
            $connection = $this->entityManager->getConnection();
                
            try {
                $connection->beginTransaction();
                
                //Pega o email pelo id, mas não marca como lido.
                $mail = $mailBox->getMail($mailId, false);
                $mailInfo = array (
                    "DATE"      => $mail->date,
                    "SUBJECT"   => empty($mail->subject) ? '<SEM ASSUNTO>' : $mail->subject,
                    "FROMNAME"  => $mail->fromName,
                    "FROMMAIL"  => $mail->fromAddress,
                    //Se não encontrar o textPlain, formata o HTML tirando as tags e múltiplos espaços.
                    "MESSAGE"   => empty($mail->textPlain) ? preg_replace("/(?:\s\s+|\n|\t)/", "\n",$this->getPlainTextFromHtml($this->deleteAllBetweenStyle($mail->textHtml))) : preg_replace("/(?:\s\s+|\n|\t)/", "\n", $mail->textPlain),
                    "ATTACH"    => !empty($mail->getAttachments()) ? $mail->getAttachments() : ''
                );
                $returnMessage = $this->ticketBotFactory->saveTicketFromMailInfo($mailInfo);
                
                if(!empty($returnMessage['TCTID'])){
                    $newParc = $this->checkIfIsNewParc($returnMessage['MESSAGE']);
                    
                    if($newParc !== false){
                        $newParc = $returnMessage['PARCNEGOCIO'];
                    }
        
                    $this->sendMailOpenedTicket($mailInfo, $returnMessage['TCTID'], $newParc);
                }
                
                $tickets[$count]['MESSAGE']     = $mailId.': '.$returnMessage['MESSAGE'];
                $successMails[] = $mailId;
                $connection->commit();
                $count++;
                
            } catch (\Exception $e) {
                
                $errorMails[]   = $mailId;
                $errorMessage[] = $mailId.': '.$e->getMessage();
                
                $connection->rollBack();
            }
            
        }
        //Marca os emails que geraram tickets como lidos e os que falharam como não lidos.
        if(!empty($errorMails)){
            $mailBox->markMailsAsUnread($errorMails);
        }if(!empty($successMails)){
            $mailBox->markMailsAsRead($successMails);
        }
        return array("ticketsCreated" => $tickets, "errors" => $errorMessage, "successMails" => $successMails);
    }
    
    /**
     *  Pega os dados relacionados a IMAP da parametrização do CRM para todas as Orgs.
     * 
     *  @return array       Array contendo o host, o username e o password decriptografado.
     *  
     *  @throws \Exception
    */
    public function getImapInfo(){
        try{
            $imapInfo = $this->ticketBotFactory->getImapInfo();
            $imapInfoOrgs = array();
            if(!empty($imapInfo)){
                if(is_object($imapInfo)){
                    
                    $nrorg = $imapInfo->getNrorg();    
                    $imapInfoOrgs[$nrorg] = $this->setImapinfo($imapInfo);
                    return $imapInfoOrgs;
                
                }else if(is_array($imapInfo)){
                    
                    foreach ($imapInfo as $org) {
                        $nrorg = $org->getNrorg();    
                        $imapInfoOrgs[$nrorg] = $this->setImapinfo($org);
                    } 
                    return $imapInfoOrgs;
                }
            }else{
                throw new \Exception('Parametrization not found in org');    
            }
            
        }catch(\Exception $e){
            throw new \Exception('Cannot get IMAP information'.' -> '.$e->getMessage());
        }
    }
    /**
     * Monta o array com as informações do IMAP
    */
    protected function setImapinfo($imapInfo){
        return array(
            'HOSTIMAP' => $imapInfo->getTxhostimap(),
            'USERIMAP' => $imapInfo->getTxuserimap(),
            'PASSIMAP' => \Teknisa\CRM\Util\Util::decrypt($imapInfo->getTxpassimap())
        );
    }
    
    /**
     *  Tira as tags HTML de uma string 
     * 
     *  @return string       Texto limpo sem as tags HTML.
     *  
    */
    private function getPlainTextFromHtml($html) {
        $html = strip_tags($html);
        $html = html_entity_decode($html, ENT_QUOTES, 'UTF-8');
        return $html;
    }
    
    /**
     *  Remove texto entre dois parâmetros                
     * 
     *  @return string       Texto de entrada, exceto o que foi encontrado entre as tags passadas como parâmetro.
     *  
    */
    private function deleteAllBetweenStyle($string) {
      $end = '</style>';
      $beginning = '<style>';
      $beginningPos = strpos($string, $beginning);
      if(!$beginningPos){
        $beginningPos = strpos($string, '<style type="text/css">');
      }
      $endPos = strpos($string, $end);
      if ($beginningPos === false || $endPos === false) {
        return $string;
      }
      $textToDelete = substr($string, $beginningPos, ($endPos + strlen($end)) - $beginningPos);
      return str_replace($textToDelete, "\n", $string);
    }
    
    
    /**
     *  Envia o email notificando a abertura do ticket                
     * 
     *  @param array            $mailInfo       Array contendo as informações do email.
     *  @param string|number    $nrseqcontato   NRSEQCONTATO do ticket.
     *  
    */
    public function sendMailOpenedTicket($mailInfo, $nrseqcontato, $newParc){
        
        $mensagem = array();
        $nrseqcontato = (int)$nrseqcontato;
        if($newParc !== false){
            // #/ pra evitar que o href codifique o #
            $mensagem['LINK'] = "https://crm.teknisa.com/#/product#/crm_profile?nrparcnegocio=".$newParc;
            $mensagem['LABELLINK'] = "para completar seu cadastro e acompanhar seu ticket";
        }else{
            $mensagem['LINK'] = "https://crm.teknisa.com/#/product#/crm_aberturachamado";
            $mensagem['LABELLINK'] = "para acessar o CRM e acompanhar seu ticket";
        }
        
        $mensagem['DSTITULOMENSAGEM'] = 'CRM - Ticket Criado - #'.$nrseqcontato;
        $mensagem['DSMENSAGEM']    = '<table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee" style="background-color: #eeeeee;"> <tbody> <tr> <td> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="0274BA" class="container590" style="background-color: #0274ba; text-align: center;"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="center" valign="middle"> <table border="0" align="middle" cellpadding="0" cellspacing="0" style="border-collapse:collapse; mso-table-lspace:0pt; mso-table-rspace:0pt;" class=""> <tbody> <tr> <td align="center" valign="middle"> <p style="color: #ffffff; font-size: 25px; padding:35px !important; font-family: Helvetica, sans-serif; line-height: 24px; ">NOTIFICAÇÃO DE TICKET</p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"></p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"></p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"></p> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> <table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee"> <tbody> <tr> <td align="center"> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="ffffff" class="container590 bg_color"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="left" style="color: #39434b; font-size: 20px; font-family: " Montserrat ", Helvetica, sans-serif; font-weight: 400; padding-bottom:15px; padding-top:20px; line-height: 35px;" class="title_color"> <div style="line-height:22px;padding:20px;"> Prezado(a) [INTERLOCUTOR]</div> </td> </tr> <tr> <td align="left" class="title_color" style="color: #848999; font-size: 14px; font-family: " Questrial ", sans-serif; mso-line-height-rule: exactly; line-height: 24px; padding-bottom:15px;"> <div style="line-height: 24px; margin-top:-5px; margin-bottom:15px;"> <p>Seu Ticket: <label>#[NRSEQCONTATO]</label> - [REFERENCIA] foi aberto com sucesso. </p> <a href="[LINK]">Clique aqui</a> <p style="display:inline">[LABELLINK] </p> <p>Agradecemos o contato e retornaremos em breve.</p> <p>Atenciosamente,</p> <p>Equipe CRM</p> </div> </td> </tr> <tr> <td height="10" style="font-size: 10px; line-height: 10px;">&nbsp;</td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table>';
        $mensagem['INTERLOCUTOR']  = $mailInfo['FROMNAME'];
        $mensagem['NRSEQCONTATO']  = $nrseqcontato;
        $mensagem['REFERENCIA']    = $mailInfo['SUBJECT'];
        $this->enviarMensagemController->enviarEmail($mensagem, $this->environment->getNrOrgTrab(), null, null, $mailInfo['FROMMAIL'], $mailInfo['FROMNAME']);
        
        
    }
    
    /**
     *  Busca na mensagem de retorno a flag que indica que um novo parceiro foi criado              
     * 
     *  @param  string          $message   Mensagem de Retorno da rotina de Inserção de Tickets.
     *  
     *  @return number|boolean             Posição da flag na mensagem de retorno ou false, se o parceiro já existir.
    */
    private function checkIfIsNewParc($message){
        return strpos($message, 'Missing Oper');
    }
    
    /**
     *  Cria o novo operador vindo do link de confirmação do email e insere no perfil de acesso padrão    
     * 
     *  @param  array          $oper   Array com as informações do novo operador
    */
    public function createOperAndRedirect($oper){
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $return =  $this->ticketBotFactory->createOper($oper);
            if($return['CDOPERADOR'])
                $this->ticketBotFactory->insertInPerf($oper);
            $connection->commit();
            return $return;

        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }    
    }
    
    /**
     *  Cria o novo interlocutor vindo da tela de ticket
     * 
     *  @param  array          $interlocInfo   Array com as informações do novo interlocutor
    */
    public function createInterloc($interlocInfo){
        $connection = $this->entityManager->getConnection();
        try {
            
            $this->ticketBotFactory->setMailInfo($interlocInfo);
            $this->ticketBotFactory->setKey(array('NRPARCNEGOCIOPARENT' => $interlocInfo['NRPARCNEGOCIOPARENT']));
            $connection->beginTransaction();
             
            $this->ticketBotFactory->createParc();
            
            $return = strpos(implode(', ', $this->ticketBotFactory->getReturnMessage()), 'CrmInterloc Created');
            
            if($return !== false){
                $connection->commit();
                return array(
                    "NMINTERLOCUTOR" => $interlocInfo['FROMNAME'],
                    "EMAILINTERLOC"  => $interlocInfo['FROMMAIL'],
                    "NRPARCNEGOCIO"  => $this->ticketBotFactory->getKeys()['NRPARCNEGOCIO']
                );
            }else{
                throw new \Exception('Interlocutor não criado.');   
            }
        
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }    
    }
    
    /**
     *  Verifica se já existe operador vinculado ao interlocutor que foi atribuido ao ticket.
     * 
     *  @param  array          $ticketInfo      Informações do novo ticket criado
     *  @param  number         $nrseqcontato    PK do novo ticket
    */
    public function prepareSendMailOpenedTicket($ticketInfo, $nrseqcontato){
        try {
            $this->ticketBotFactory->setMailInfo($ticketInfo);
            $newParc;
            $operador = $this->ticketBotFactory->findInEntity('\Teknisa\Model\Geral\Entities\Operador', 'dsemailoper');
            if(is_object($operador)){
                $newParc = false;
            }else{
                $newParc = $ticketInfo['NRPARCINTERLOCUTOR'];
            }
            
            $this->sendMailOpenedTicket($ticketInfo, $nrseqcontato, $newParc);
        
        } catch (\Exception $e) {
            throw $e;
        }    
    }
    
}